﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetWarshipsGeneric<T>
        where T : class
    {
        /// <summary>
		/// Массив объектов, которые храним
		/// </summary>
		private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetWarshipsGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="warship">Добавляемый военный корабль</param>
        /// <returns></returns>
        public int Insert(T warship)
        {
            return Insert(warship, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="warship">Добавляемый военный корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T warship, int position)
        {
            int EmptyElement = -1;
            if (position >= Count || position < 0) return -1;

            if (_places[position] == null)
            {
                _places[position] = warship;
                return 1;
            }

            else if (_places[position] != null)
            {
                for (int i = position + 1; i < Count; i++)
                    if (_places[i] == null)
                    {
                        EmptyElement = i;
                        break;
                    }

                if (EmptyElement == -1)
                    return -1;

                for (int i = EmptyElement; i > position; i--)
                    _places[i] = _places[i - 1];
            }

            _places[position] = warship;
            return 1;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position >= Count || position < 0 || _places[position] == null)
                return null;

            T deleted = _places[position];
            _places[position] = null;
            return deleted;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position >= _places.Length) return null;
            return _places[position];
        }
    }
}
